package
{

	import SGComponents.*;

	import away3d.cameras.*;
	import away3d.containers.*;
	import away3d.core.base.*;
	import away3d.core.math.*;
	import away3d.core.utils.*;
	import away3d.events.*;
	import away3d.loaders.*;
	import away3d.materials.*;
	import away3d.primitives.*;

	import flash.display.*;
	import flash.events.*;

	[SWF(backgroundColor="#ffffff", frameRate="30", quality="LOW", width="570", height="250")]
	public class anim2 extends Sprite implements ICheckBox
	{

		//engine variables
		private var _Camera:HoverCamera3D;
		private var _View:View3D;

		// camera
		private var _LastPanAngle:Number;
		private var _LastTiltAngle:Number;
		private var _LastMouseX:Number;
		private var _LastMouseY:Number;
		private var _Moving:Boolean;

		//objects
		private var _Torus:Torus;
		private var _Sphere:Sphere;
		private var _Cylinder:Cylinder;
		private var _Plane:Plane;
		private var _Mesh:Mesh;


		//checkboxes
		private var _Checkbox1:CheckBox;
		private var _Checkbox2:CheckBox;
		private var _Checkbox3:CheckBox;

		//materials
		private var _ColorMaterial:ColorMaterial;
		private var _WireFrameMaterial:WireframeMaterial;

		public function anim2()
		{
			initEngine();
			initMaterial();
			initObjects();
			initSliders();
			initListeners();
		}

		private function initMaterial():void
		{
			_ColorMaterial=new ColorMaterial();
			_ColorMaterial.color=0xF4661B;

			_WireFrameMaterial=new WireframeMaterial();
			_WireFrameMaterial.wireColor=0x000000;
			_WireFrameMaterial.thickness=3;

		}

		private function initEngine():void
		{
			_Camera=new HoverCamera3D({zoom: 6, focus: 100, distance: 700});

			_View=new View3D({camera: _Camera, x: stage.stageWidth * .4, y: stage.stageHeight * .5});
			addChild(_View);
			_View.forceUpdate=true;

			_Camera.panAngle=-120;
			_Camera.tiltAngle=9.5;
			_Camera.minTiltAngle=-90;
			_Camera.hover(true);

		}

		private function initSliders():void
		{

			CheckBox.X_POS=540;
			CheckBox.Y_POS=20;
			CheckBox.Y_OFFSET=20;
			CheckBox.CALLBACK_INTERFACE=this;
			CheckBox.FONT_SIZE=12;

			_Checkbox1=new CheckBox(this, true);
			_Checkbox1.setTitle("torus");
			_Checkbox1.setOrientation(CheckBox.RIGHT);
			_Checkbox1.create();
			_Checkbox1.setState(true);

			_Checkbox2=new CheckBox(this, true);
			_Checkbox2.setTitle("sphere");
			_Checkbox2.setOrientation(CheckBox.RIGHT);
			_Checkbox2.create();
			_Checkbox2.setState(false);

			_Checkbox3=new CheckBox(this, true);
			_Checkbox3.setTitle("cylindre");
			_Checkbox3.setOrientation(CheckBox.RIGHT);
			_Checkbox3.create();
			_Checkbox3.setState(false);

		}

		private function createTorus():Mesh
		{
			_Torus=new Torus();
			_Torus.scale(1.3);
			
			_Torus.material=_ColorMaterial;
			_Torus.outline=_WireFrameMaterial;

			return _Torus as Mesh;
		}

		private function createCylinder():Mesh
		{
			_Cylinder=new Cylinder();
			_Cylinder.radius=140;
			_Cylinder.height=170;
			_Cylinder.segmentsH=15;
			_Cylinder.segmentsW=15;

			_Cylinder.material=_ColorMaterial;
			_Cylinder.outline=_WireFrameMaterial;

			return _Cylinder as Mesh;
		}

		private function createSphere():Mesh
		{
			_Sphere=new Sphere();
			_Sphere.radius=140;

			_Sphere.material=_ColorMaterial;
			_Sphere.outline=_WireFrameMaterial;

			return _Sphere as Mesh;
		}

		private function initObjects():void
		{
			_Mesh=this.createTorus();
			_View.scene.addChild(_Mesh);
		}

		private function initListeners():void
		{
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
		}

		private function onMouseWheel(event:MouseEvent):void
		{

			_Camera.distance+=-event.delta * 10;

			if (_Camera.distance < 50)
			{
				_Camera.distance=50;
			}

		}

		private function onMouseDown(event:MouseEvent):void
		{
			_LastPanAngle=_Camera.panAngle;
			_LastTiltAngle=_Camera.tiltAngle;

			_LastMouseX=stage.mouseX;
			_LastMouseY=stage.mouseY;
			_Moving=true;

			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}

		private function onMouseUp(event:MouseEvent):void
		{
			_Moving=false;
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}

		private function onEnterFrame(event:Event):void
		{
			var cameraSpeed:Number=0.5;

			if (_Moving)
			{
				_Camera.panAngle=0.5 * (stage.mouseX - _LastMouseX) + _LastPanAngle;
				_Camera.tiltAngle=0.5 * (stage.mouseY - _LastMouseY) + _LastTiltAngle;
			}

			_Camera.hover();
			_View.render();
		}

		public function onCheckBoxStateChange(ThisCheckBox:CheckBox):void
		{
			if (_Mesh)
			{
				_View.scene.removeChild(_Mesh);
				_Mesh=null;
			}

			_Torus=null;
			_Sphere=null;
			_Cylinder=null;
			_Plane=null;

			switch (ThisCheckBox)
			{
				case _Checkbox1: //cube					
					_Mesh=this.createTorus();
					_View.scene.addChild(_Mesh);
					break;
				case _Checkbox2: //sphere
					_Mesh=this.createSphere();
					_View.scene.addChild(_Mesh);

					break;
				case _Checkbox3: //cylindre
					_Mesh=this.createCylinder();
					_View.scene.addChild(_Mesh);
					break;

			}

		}

	}
}