package
{

	import SGComponents.*;

	import away3d.cameras.*;
	import away3d.containers.*;
	import away3d.core.base.*;
	import away3d.core.math.*;
	import away3d.core.utils.*;
	import away3d.events.*;
	import away3d.loaders.*;
	import away3d.materials.BitmapMaterial;
	import away3d.primitives.*;

	import flash.display.*;
	import flash.events.*;

	[SWF(backgroundColor="#ffffff", frameRate="30", quality="LOW", width="570", height="250")]
	public class anim4 extends Sprite implements ISlider, ICheckBox
	{
		[Embed(source="/assets/damier.jpg")]
		private var _Texture:Class;

		//engine variables
		private var _Camera:HoverCamera3D;
		private var _View:View3D;

		// camera
		private var _LastPanAngle:Number;
		private var _LastTiltAngle:Number;
		private var _LastMouseX:Number;
		private var _LastMouseY:Number;
		private var _Moving:Boolean;

		//objects
		private var _Cube:Cube;
		private var _Sphere:Sphere;
		private var _Cylinder:Cylinder;
		private var _Plane:Plane;
		private var _Mesh:Mesh;

		// Sliders		
		private var _Slider1:Slider;

		//checkboxes
		private var _Checkbox1:CheckBox;
		private var _Checkbox2:CheckBox;
		private var _Checkbox3:CheckBox;
		private var _Checkbox4:CheckBox;

		//materials
		private var _BitmapMaterial:BitmapMaterial;

		private var _Smooth:Boolean=false;

		public function anim4()
		{
			initEngine();
			initMaterial();
			initObjects();
			initSliders();
			initListeners();
		}

		private function initMaterial():void
		{
			var TextureBitmap:Bitmap=new _Texture() as Bitmap;
			_BitmapMaterial=new BitmapMaterial(Cast.bitmap(TextureBitmap));

			_BitmapMaterial.smooth=_Smooth;
		}

		private function initEngine():void
		{
			_Camera=new HoverCamera3D({zoom: 6, focus: 100, distance: 700});

			_View=new View3D({camera: _Camera, x: stage.stageWidth * .5, y: stage.stageHeight * .5});
			addChild(_View);

			_Camera.panAngle=-120;
			_Camera.tiltAngle=9.5;
			_Camera.minTiltAngle=-90;
			_Camera.hover(true);

		}

		private function initSliders():void
		{
			Slider.CALLBACK_INTERFACE=this;
			Slider.SLIDER_CONTAINER=stage;
			Slider.CURSOR_HEIGH=10;
			Slider.CURSOR_WIDTH=8;
			Slider.CONTROL_WIDTH=80;
			Slider.LINE_THICKNESS=3;
			Slider.FONT_SIZE=12;
			Slider.X_POS=20;
			Slider.Y_POS=40;
			Slider.Y_OFFSET=40;
			Slider.STICKY=true;

			_Slider1=new Slider();
			_Slider1.setWidth(20);
			_Slider1.setMinMax(0, 1);
			_Slider1.setTitle("smooth");
			_Slider1.create();
			_Slider1.setCursorPosition(0);

			CheckBox.X_POS=540;
			CheckBox.Y_POS=20;
			CheckBox.Y_OFFSET=20;
			CheckBox.CALLBACK_INTERFACE=this;
			CheckBox.FONT_SIZE=12;

			_Checkbox1=new CheckBox(this, true);
			_Checkbox1.setTitle("cube");
			_Checkbox1.setOrientation(CheckBox.RIGHT);
			_Checkbox1.create();
			_Checkbox1.setState(true);

			_Checkbox2=new CheckBox(this, true);
			_Checkbox2.setTitle("plane");
			_Checkbox2.setOrientation(CheckBox.RIGHT);
			_Checkbox2.create();
			_Checkbox2.setState(false);

			_Checkbox3=new CheckBox(this, true);
			_Checkbox3.setTitle("sphere");
			_Checkbox3.setOrientation(CheckBox.RIGHT);
			_Checkbox3.create();
			_Checkbox3.setState(false);

			_Checkbox4=new CheckBox(this, true);
			_Checkbox4.setTitle("cylindre");
			_Checkbox4.setOrientation(CheckBox.RIGHT);
			_Checkbox4.create();
			_Checkbox4.setState(false);

		}

		private function createCube():Mesh
		{
			_Cube=new Cube();
			_Cube.width=180;
			_Cube.height=180;
			_Cube.depth=180;

			_Cube.material=_BitmapMaterial;
			_Cube.mappingType=CubeMappingType.MAP6;
			_BitmapMaterial.repeat=false;
			return _Cube as Mesh;
		}

		private function createCylinder():Mesh
		{
			_Cylinder=new Cylinder();
			_Cylinder.radius=140;
			_Cylinder.height=170;
			_Cylinder.segmentsH=10;
			_Cylinder.segmentsW=10;

			_Cylinder.material=_BitmapMaterial;

			return _Cylinder as Mesh;
		}

		private function createPlane():Mesh
		{
			_Plane=new Plane();
			_Plane.width=190;
			_Plane.height=190;
			_Plane.segmentsH=1;
			_Plane.rotationX=-90;
			_Plane.bothsides=true;

			_Plane.material=_BitmapMaterial;
			return _Plane as Mesh;
		}

		private function createSphere():Mesh
		{
			_Sphere=new Sphere();
			_Sphere.radius=140;
			_Sphere.segmentsH=10;
			_Sphere.segmentsW=10;

			_Sphere.material=_BitmapMaterial;

			return _Sphere as Mesh;
		}

		public function updateMeshProperties(ThisSlider:Slider):void
		{
			_Smooth=ThisSlider.getCursorPosition() == 1;
			_BitmapMaterial.smooth=_Smooth;
		}

		private function initObjects():void
		{
			_Mesh=this.createCube();
			_View.scene.addChild(_Mesh);
		}

		private function initListeners():void
		{
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
		}

		private function onMouseWheel(event:MouseEvent):void
		{

			_Camera.distance+=-event.delta * 10;

			if (_Camera.distance < 50)
			{
				_Camera.distance=50;
			}

		}

		private function onMouseDown(event:MouseEvent):void
		{
			_LastPanAngle=_Camera.panAngle;
			_LastTiltAngle=_Camera.tiltAngle;

			_LastMouseX=stage.mouseX;
			_LastMouseY=stage.mouseY;
			_Moving=true;

			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}

		private function onMouseUp(event:MouseEvent):void
		{
			_Moving=false;
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}

		private function onEnterFrame(event:Event):void
		{

			if (_Moving)
			{
				_Camera.panAngle=0.5 * (stage.mouseX - _LastMouseX) + _LastPanAngle;
				_Camera.tiltAngle=0.5 * (stage.mouseY - _LastMouseY) + _LastTiltAngle;
			}

			_Camera.hover();
			_View.render();
		}

		// override ISlider's properties
		public function onSliderMove(ThisSlider:Slider):void
		{
			updateMeshProperties(ThisSlider);
		}

		// override ISlider's properties
		public function onSliderRelease(ThisSlider:Slider):void
		{
			//	updateMeshProperties(ThisSlider);
		}

		public function onCheckBoxStateChange(ThisCheckBox:CheckBox):void
		{
			if (_Mesh)
			{
				_View.scene.removeChild(_Mesh);
				_Mesh=null;
			}

			_Cube=null;
			_Sphere=null;
			_Cylinder=null;
			_Plane=null;

			switch (ThisCheckBox)
			{
				case _Checkbox1: //cube					
					_Mesh=this.createCube();
					_View.scene.addChild(_Mesh);
					break;
				case _Checkbox2: //plane
					_Mesh=this.createPlane();
					_View.scene.addChild(_Mesh);
					break;
				case _Checkbox3: //sphere
					_Mesh=this.createSphere();
					_View.scene.addChild(_Mesh);

					break;
				case _Checkbox4: //cylindre
					_Mesh=this.createCylinder();
					_View.scene.addChild(_Mesh);
					break;

			}

		}

	}
}