package
{

	import SGComponents.*;

	import away3d.cameras.*;
	import away3d.containers.*;
	import away3d.core.base.*;
	import away3d.core.math.*;
	import away3d.core.utils.*;
	import away3d.events.*;
	import away3d.lights.*;
	import away3d.loaders.*;
	import away3d.materials.*;
	import away3d.primitives.*;

	import flash.display.*;
	import flash.events.*;

	[SWF(backgroundColor="#000000", frameRate="30", quality="LOW", width="570", height="280")]
	public class anim3 extends Sprite implements ISlider, ICheckBox
	{
		[Embed(source="/assets/damier.jpg")]
		private var _Texture:Class;

		//engine variables
		private var _Camera:HoverCamera3D;
		private var _View:View3D;

		// camera
		private var _LastPanAngle:Number;
		private var _LastTiltAngle:Number;
		private var _LastMouseX:Number;
		private var _LastMouseY:Number;
		private var _Moving:Boolean;

		// light
		private var _Light:DirectionalLight3D;
		private var _LightRot:Number=0;
		private var _LightAmbient:Number=0.1;
		private var _LightDiffuse:Number=0.4;
		private var _LightSpecular:Number=0.1;
		private var _LightBrightness:Number=1;

		//objects
		private var _Torus:Torus;
		private var _Sphere:Sphere;
		private var _Cylinder:Cylinder;
		private var _Mesh:Mesh;

		// Sliders		
		private var _Slider1:Slider;
		private var _Slider2:Slider;
		private var _Slider3:Slider;
		private var _Slider4:Slider;
		private var _Slider5:Slider;

		//checkboxes
		private var _Checkbox1:CheckBox;
		private var _Checkbox2:CheckBox;
		private var _Checkbox3:CheckBox;

		//materials
		private var _PhongBitmapMaterial:PhongBitmapMaterial;
		private var _MaterialShininess:Number=20;
		private var _MaterialSpecular:int=0xffffff;

		public function anim3()
		{
			initEngine();
			initObjects();
			initSliders();
			initListeners();
		}

		private function initEngine():void
		{
			_Camera=new HoverCamera3D({zoom: 6, focus: 100, distance: 700});

			_View=new View3D({camera: _Camera, x: stage.stageWidth * .5, y: stage.stageHeight * .5});
			addChild(_View);

			_Camera.panAngle=-120;
			_Camera.tiltAngle=9.5;
			_Camera.minTiltAngle=-90;
			_Camera.hover(true);
		}

		private function initSliders():void
		{
			Slider.CALLBACK_INTERFACE=this;
			Slider.SLIDER_CONTAINER=stage;
			Slider.CURSOR_HEIGH=10;
			Slider.CURSOR_WIDTH=8;
			Slider.CONTROL_WIDTH=80;
			Slider.LINE_THICKNESS=3;
			Slider.FONT_SIZE=12;
			Slider.X_POS=20;
			Slider.Y_POS=40;
			Slider.Y_OFFSET=40;
			Slider.STICKY=true;
			Slider.TEXT_COLOR=0x888888;
			Slider.LINE_COLOR=0x888888;
			Slider.CURSOR_COLOR=0xaaaaaa;

			_Slider1=new Slider();
			_Slider1.setMinMax(0, 1, 1);
			_Slider1.setTitle("ambient");
			_Slider1.create();
			_Slider1.setCursorPosition(_LightAmbient);

			_Slider2=new Slider();
			_Slider2.setMinMax(0, 1, 1);
			_Slider2.setTitle("diffuse");
			_Slider2.create();
			_Slider2.setCursorPosition(_LightDiffuse);

			_Slider3=new Slider();
			_Slider3.setMinMax(0, 1, 1);
			_Slider3.setTitle("specular");
			_Slider3.create();
			_Slider3.setCursorPosition(_LightSpecular);

			_Slider4=new Slider();
			_Slider4.setMinMax(0, 1, 1);
			_Slider4.setTitle("brightness");
			_Slider4.create();
			_Slider4.setCursorPosition(_LightBrightness);

			_Slider5=new Slider();
			_Slider5.setMinMax(0, 40);
			_Slider5.setTitle("shininess");
			_Slider5.create();
			_Slider5.setCursorPosition(_MaterialShininess);

			CheckBox.X_POS=530;
			CheckBox.Y_POS=20;
			CheckBox.Y_OFFSET=20;
			CheckBox.CALLBACK_INTERFACE=this;
			CheckBox.FONT_SIZE=12;
			CheckBox.SQUARE_COLOR=0x888888;
			CheckBox.TICK_COLOR=0xaaaaaa;
			CheckBox.TITLE_COLOR=0x888888;

			_Checkbox2=new CheckBox(this, true);
			_Checkbox2.setTitle("sphere");
			_Checkbox2.setOrientation(CheckBox.RIGHT);
			_Checkbox2.create();
			_Checkbox2.setState(true);

			_Checkbox3=new CheckBox(this, true);
			_Checkbox3.setTitle("cylindre");
			_Checkbox3.setOrientation(CheckBox.RIGHT);
			_Checkbox3.create();
			_Checkbox3.setState(false);

			_Checkbox1=new CheckBox(this, true);
			_Checkbox1.setTitle("torus");
			_Checkbox1.setOrientation(CheckBox.RIGHT);
			_Checkbox1.create();
			_Checkbox1.setState(false);
		}

		private function createTorus():Mesh
		{
			_Torus=new Torus();

			var TextureBitmap:Bitmap=new _Texture() as Bitmap;
			_PhongBitmapMaterial=new PhongBitmapMaterial(Cast.bitmap(TextureBitmap));
			_PhongBitmapMaterial.shininess=_MaterialShininess;
			_PhongBitmapMaterial.specular=_MaterialSpecular;

			_Torus.material=_PhongBitmapMaterial;

			return _Torus as Mesh;
		}

		private function createCylinder():Mesh
		{
			_Cylinder=new Cylinder();
			_Cylinder.radius=100;
			_Cylinder.height=170;
			_Cylinder.segmentsH=15;
			_Cylinder.segmentsW=15;

			var TextureBitmap:Bitmap=new _Texture() as Bitmap;
			_PhongBitmapMaterial=new PhongBitmapMaterial(Cast.bitmap(TextureBitmap));
			_PhongBitmapMaterial.shininess=_MaterialShininess;
			_PhongBitmapMaterial.specular=_MaterialSpecular;
			_Cylinder.material=_PhongBitmapMaterial;

			return _Cylinder as Mesh;
		}

		private function createSphere():Mesh
		{
			_Sphere=new Sphere();
			_Sphere.radius=100;
			_Sphere.segmentsH=10;
			_Sphere.segmentsW=10;

			var TextureBitmap:Bitmap=new _Texture() as Bitmap;
			_PhongBitmapMaterial=new PhongBitmapMaterial(Cast.bitmap(TextureBitmap));
			_PhongBitmapMaterial.shininess=_MaterialShininess;
			_PhongBitmapMaterial.specular=_MaterialSpecular;
			_Sphere.material=_PhongBitmapMaterial;

			return _Sphere as Mesh;
		}

		public function updateMeshProperties(ThisSlider:Slider):void
		{
			switch (ThisSlider)
			{
				case _Slider1: // ambient
					_LightAmbient=ThisSlider.getCursorPosition();
					_Light.ambient=_LightAmbient
					break;
				case _Slider2: //diffuse		
					_LightDiffuse=ThisSlider.getCursorPosition();
					_Light.diffuse=_LightDiffuse
					break;

				case _Slider3: //specular
					_LightSpecular=ThisSlider.getCursorPosition();
					_Light.specular=_LightSpecular
					break;

				case _Slider4: //brightness
					_LightBrightness=ThisSlider.getCursorPosition();
					_Light.brightness=_LightBrightness
					break;

				case _Slider5:
					_MaterialShininess=ThisSlider.getCursorPosition();
					_PhongBitmapMaterial.shininess=_MaterialShininess;
					break;
			}
		}

		private function initObjects():void
		{

			_Light=new DirectionalLight3D({color: 0xFFFFFF, ambient: _LightAmbient, diffuse: _LightDiffuse, specular: _LightSpecular, brightness: _LightBrightness});
			_View.scene.addLight(_Light);

			_Mesh=this.createSphere();
			_View.scene.addChild(_Mesh);
		}

		private function initListeners():void
		{
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
		}

		private function onMouseWheel(event:MouseEvent):void
		{

			_Camera.distance+=-event.delta * 10;

			if (_Camera.distance < 50)
			{
				_Camera.distance=50;
			}

		}

		private function onMouseDown(event:MouseEvent):void
		{
			_LastPanAngle=_Camera.panAngle;
			_LastTiltAngle=_Camera.tiltAngle;

			_LastMouseX=stage.mouseX;
			_LastMouseY=stage.mouseY;
			_Moving=true;

		}

		private function onMouseUp(event:MouseEvent):void
		{
			_Moving=false;
		}

		private function onEnterFrame(event:Event):void
		{
			var cameraSpeed:Number=0.5;

			if (_Moving)
			{
				_Camera.panAngle=cameraSpeed * (stage.mouseX - _LastMouseX) + _LastPanAngle;
				_Camera.tiltAngle=cameraSpeed * (stage.mouseY - _LastMouseY) + _LastTiltAngle;
			}

			_LightRot++;
			_Light.direction.x=1;
			_View.forceUpdate=true;
			_Camera.hover();
			_View.render();
		}

		public function onSliderMove(ThisSlider:Slider):void
		{
			updateMeshProperties(ThisSlider);
		}

		public function onSliderRelease(ThisSlider:Slider):void
		{
		}

		public function onCheckBoxStateChange(ThisCheckBox:CheckBox):void
		{
			if (_Mesh)
			{
				_View.scene.removeChild(_Mesh);
				_Mesh=null;
			}

			_Torus=null;
			_Sphere=null;
			_Cylinder=null;

			switch (ThisCheckBox)
			{
				case _Checkbox1: //cube					
					_Mesh=this.createTorus();
					_View.scene.addChild(_Mesh);
					break;
				case _Checkbox2: //sphere
					_Mesh=this.createSphere();
					_View.scene.addChild(_Mesh);

					break;
				case _Checkbox3: //cylindre
					_Mesh=this.createCylinder();
					_View.scene.addChild(_Mesh);
					break;
			}
		}
	}
}