package
{

	import SGComponents.*;

	import away3d.cameras.*;
	import away3d.containers.*;
	import away3d.core.base.*;
	import away3d.core.math.*;
	import away3d.core.utils.*;
	import away3d.events.*;
	import away3d.loaders.*;
	import away3d.materials.*;
	import away3d.primitives.*;

	import flash.display.*;
	import flash.events.*;

	[SWF(backgroundColor="#000000", frameRate="30", quality="LOW", width="570", height="280")]
	public class anim7 extends Sprite implements ISlider, ICheckBox
	{

		[Embed(source="assets/damier.jpg")]
		private var _Texture:Class;

		[Embed(source="assets/fisheye.jpg")]
		private var _Reflective:Class;

		//engine variables
		private var _Camera:HoverCamera3D;
		private var _View:View3D;

		// camera
		private var _LastPanAngle:Number;
		private var _LastTiltAngle:Number;
		private var _LastMouseX:Number;
		private var _LastMouseY:Number;
		private var _Moving:Boolean;

		//objects
		private var _Cube:Cube;
		private var _Sphere:Sphere;
		private var _Mesh:Mesh;

		// Sliders		
		private var _Slider1:Slider;

		//checkboxes
		private var _Checkbox1:CheckBox;
		private var _Checkbox2:CheckBox;

		//materials
		private var _EnviroBitmapMaterial:EnviroBitmapMaterial;
		private var _Reflectiveness:Number=0.5;

		private var _SegmentsH:Number=10;
		private var _SegmentsW:Number=10;
		private var _Smooth:Boolean=false;

		public function anim7()
		{
			initEngine();
			initMaterial();
			initObjects();
			initSliders();
			initListeners();
		}

		private function initMaterial():void
		{
			var TextureBitmap:Bitmap=new _Texture() as Bitmap;
			var ReflectiveBitmap:Bitmap=new _Reflective() as Bitmap;

			_EnviroBitmapMaterial=new EnviroBitmapMaterial(Cast.bitmap(TextureBitmap), Cast.bitmap(ReflectiveBitmap));
			_EnviroBitmapMaterial.reflectiveness=_Reflectiveness;
			_EnviroBitmapMaterial.smooth=_Smooth;
		}

		private function initEngine():void
		{
			_Camera=new HoverCamera3D({zoom: 6, focus: 100, distance: 600});

			_View=new View3D({camera: _Camera, x: stage.stageWidth * .5, y: stage.stageHeight * .5});
			addChild(_View);

			_Camera.panAngle=_Camera.panAngle=-120;
			_Camera.tiltAngle=_Camera.tiltAngle=9.5;
			_Camera.minTiltAngle=-90;
			_Camera.hover(true);

		}

		private function initSliders():void
		{
			Slider.CALLBACK_INTERFACE=this;
			Slider.SLIDER_CONTAINER=stage;
			Slider.CURSOR_HEIGH=10;
			Slider.CURSOR_WIDTH=8;
			Slider.CONTROL_WIDTH=80;
			Slider.LINE_THICKNESS=3;
			Slider.FONT_SIZE=12;
			Slider.X_POS=20;
			Slider.Y_POS=40;
			Slider.Y_OFFSET=40;
			Slider.STICKY=true;
			Slider.TEXT_COLOR=0x888888;
			Slider.LINE_COLOR=0x888888;
			Slider.CURSOR_COLOR=0xaaaaaa;

			_Slider1=new Slider();
			_Slider1.setMinMax(0, 1, 1);
			_Slider1.setTitle("reflectiveness");
			_Slider1.create();
			_Slider1.setCursorPosition(_Reflectiveness);

			CheckBox.X_POS=530;
			CheckBox.Y_POS=20;
			CheckBox.Y_OFFSET=20;
			CheckBox.CALLBACK_INTERFACE=this;
			CheckBox.FONT_SIZE=12;
			CheckBox.SQUARE_COLOR=0x888888;
			CheckBox.TICK_COLOR=0xaaaaaa;
			CheckBox.TITLE_COLOR=0x888888;

			_Checkbox1=new CheckBox(this, true);
			_Checkbox1.setTitle("cube");
			_Checkbox1.setOrientation(CheckBox.RIGHT);
			_Checkbox1.create();
			_Checkbox1.setState(false);

			_Checkbox2=new CheckBox(this, true);
			_Checkbox2.setTitle("sphere");
			_Checkbox2.setOrientation(CheckBox.RIGHT);
			_Checkbox2.create();
			_Checkbox2.setState(true);

		}

		private function createCube():Mesh
		{
			_Cube=new Cube();
			_Cube.width=160;
			_Cube.height=160;
			_Cube.depth=160;
			_Cube.segmentsH=1;
			_Cube.segmentsW=1;

			_Cube.material=_EnviroBitmapMaterial;

			return _Cube as Mesh;
		}

		private function createSphere():Mesh
		{
			_Sphere=new Sphere();
			_Sphere.radius=100;
			_Sphere.segmentsH=_SegmentsH;
			_Sphere.segmentsW=_SegmentsW;

			_Sphere.material=_EnviroBitmapMaterial;

			return _Sphere as Mesh;
		}

		public function updateMeshProperties(ThisSlider:Slider):void
		{
			_Reflectiveness=ThisSlider.getCursorPosition();
			_EnviroBitmapMaterial.reflectiveness=_Reflectiveness
		}

		private function initObjects():void
		{
			_Mesh=this.createSphere();
			_View.scene.addChild(_Mesh);
		}

		private function initListeners():void
		{
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
		}

		private function onMouseWheel(event:MouseEvent):void
		{
			_Camera.distance+=-event.delta * 10;

			if (_Camera.distance < 50)
			{
				_Camera.distance=50;
			}
		}

		private function onMouseDown(event:MouseEvent):void
		{
			_LastPanAngle=_Camera.panAngle;
			_LastTiltAngle=_Camera.tiltAngle;

			_LastMouseX=stage.mouseX;
			_LastMouseY=stage.mouseY;
			_Moving=true;

		}

		private function onMouseUp(event:MouseEvent):void
		{
			_Moving=false;
		}

		private function onEnterFrame(event:Event):void
		{
			var cameraSpeed:Number=0.5;

			if (_Moving)
			{
				_Camera.panAngle=cameraSpeed * (stage.mouseX - _LastMouseX) + _LastPanAngle;
				_Camera.tiltAngle=cameraSpeed * (stage.mouseY - _LastMouseY) + _LastTiltAngle;
			}

			_Camera.hover();
			_View.render();
		}

		public function onSliderMove(ThisSlider:Slider):void
		{
			updateMeshProperties(ThisSlider);
		}

		public function onSliderRelease(ThisSlider:Slider):void
		{
		}

		public function onCheckBoxStateChange(ThisCheckBox:CheckBox):void
		{
			if (_Mesh)
			{
				_View.scene.removeChild(_Mesh);
				_Mesh=null;
			}

			_Cube=null;
			_Sphere=null;

			switch (ThisCheckBox)
			{
				case _Checkbox1: //cube					
					_Mesh=this.createCube();
					_View.scene.addChild(_Mesh);
					break;
				case _Checkbox2: //sphere
					_Mesh=this.createSphere();
					_View.scene.addChild(_Mesh);

					break;
			}
		}
	}
}